

Imports System.Collections.Generic
Imports System.Text
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.Drawing.Text



Public Class AnalogMeter
    Inherits PictureBox

    Public Sub New()
        BackColor = Color.White
        SetStyle(ControlStyles.AllPaintingInWmPaint, True)
        SetStyle(ControlStyles.UserPaint, True)
    End Sub

    Protected Overloads Overrides Sub OnResize(ByVal e As EventArgs)
        MyBase.OnResize(e)
        m_BackgroundChanged = True
    End Sub

    Protected Overloads Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        MyBase.OnPaint(e)
        DrawMeter(e.Graphics)
    End Sub

    Private Sub DrawMeter(ByVal g As Graphics)
        If m_BackgroundChanged Then CreateBackground()

        If bgImage Is Nothing Then Return

        g.DrawImageUnscaled(bgImage, 0, 0)

        g.SmoothingMode = SmoothingMode.HighQuality
        DrawLine(Value, g, p1y - 2, New Pen(m_PointerColor, Scaled_PointerWidth), , 0.08)
        'DrawLine(Value, g, 0, New Pen(m_PointerColor, Scaled_PointerWidth), , 0.02)
    End Sub



    ' ==========================================================================================================
    '  ASPECT RATIO
    ' ==========================================================================================================

    Private AspectRatio_Current As Double
    Private AspectRatio_Locked As Boolean = False

    <Description("When true the aspect ratio of the control is locked"), Category("Behavior")> _
    Public Property AspectLocked() As Boolean
        Get
            Return AspectRatio_Locked
        End Get
        Set(ByVal value As Boolean)
            AspectRatio_Locked = value
        End Set
    End Property

    Private working As Boolean = False
    Protected Overrides Sub OnSizeChanged(ByVal e As EventArgs)
        MyBase.OnSizeChanged(e)
        If Not AspectRatio_Locked Or AspectRatio_Current = 0 Then
            Me.AspectRatio_Current = Me.Height / Me.Width
        Else
            ' ------------------------------------------------------------- change height
            Height = CInt(Width * AspectRatio_Current)
            'Me.Parent.Height = Me.Height + 80
            ' ------------------------------------------------------------- change width
            'Width = CInt(Height / AspectRatio_Current)
            ' ------------------------------------------------------------- change both ( not working )
            'Dim newaspect As Double = Height / Width
            'If newaspect * 1.001 < AspectRatio_Current Then
            '    Width = CInt(Height / AspectRatio_Current)
            'ElseIf newaspect * 0.999 > AspectRatio_Current Then
            '    Height = CInt(Width * AspectRatio_Current)
            'End If
        End If
    End Sub




    ' ==========================================================================================================
    '  BACKGROUND BITMAP
    ' ==========================================================================================================

    Private bgImage As Bitmap = Nothing
    Private m_BackgroundChanged As Boolean = False
    Private p1x As Single
    Private p1y As Single

    Private Scaled_Font As Font
    Private Scaled_TickLargeSize As Single
    Private Scaled_TickLargeWidth As Single
    Private Scaled_TickSmallSize As Single
    Private Scaled_TickSmallWidth As Single
    Private Scaled_TickTinySize As Single
    Private Scaled_TickTinyWidth As Single
    Private Scaled_PointerWidth As Single


    Private Sub CreateBackground()

        If Width < 1 OrElse Height < 1 Then Return

        'Dim h As Integer
        'Dim x As Integer
        'Dim y As Integer
        'Dim w As Integer
        'Dim r As Double = Width / Height
        'If (r > 1) Then
        '    w = SizeX
        '    h = CInt(SizeY / r)
        '    x = 0
        '    y = (SizeY - h) \ 2
        'Else
        '    w = CInt(SizeX * r)
        '    h = SizeY
        '    y = 0
        '    x = (SizeX - w) \ 2
        'End If

        Dim i As Single
        Dim bmp As New Bitmap(Width, Height)
        Dim g As Graphics = Graphics.FromImage(bmp)
        g.SmoothingMode = SmoothingMode.HighQuality
        g.InterpolationMode = InterpolationMode.HighQualityBicubic
        g.TextRenderingHint = TextRenderingHint.AntiAliasGridFit

        ' ----------------------------------------------------------------- setup a clip rectangle for the meter itself
        Dim meterLocation As Rectangle
        meterLocation = New Rectangle(m_framePadding.Left, _
                                      m_framePadding.Top, _
                                      Width - m_framePadding.Horizontal, _
                                      m_internalPadding.Vertical + Width \ 2 - m_framePadding.Top)

        ' ----------------------------------------------------------------- proportional scale
        Dim kscale As Single = meterLocation.Height / 90.0F

        Scaled_Font = New Font(m_ScaleFont.Name, m_ScaleFont.Size * kscale, m_ScaleFont.Style)
        Scaled_TickLargeSize = m_TickLargeSize * kscale
        Scaled_TickLargeWidth = m_TickLargeWidth * kscale
        Scaled_TickSmallSize = m_TickSmallSize * kscale
        Scaled_TickSmallWidth = m_TickSmallWidth * kscale
        Scaled_TickTinySize = m_TickTinySize * kscale
        Scaled_TickTinyWidth = m_TickTinyWidth * kscale
        Scaled_PointerWidth = Math.Min(m_PointerWidth * kscale, 4)

        p1x = CSng((meterLocation.Width - m_internalPadding.Horizontal)) / 2
        p1y = CSng((meterLocation.Height - m_internalPadding.Vertical))

        Dim p As Pen

        If mScaleLinear Then

            ' ----------------------------------------------------------------- large ticks and numbers
            If m_TickLargeFrequency > 0 Then
                p = New Pen(m_ScaleTextColor, Scaled_TickLargeWidth)
                For i = m_MinValue To m_MaxValue Step m_TickLargeFrequency
                    DrawLine(i, g, Scaled_TickLargeSize, p, True)
                Next
            End If
            ' ----------------------------------------------------------------- small ticks
            If m_TickSmallFrequency > 0 Then
                p = New Pen(ForeColor, Scaled_TickSmallWidth)
                For i = m_MinValue To m_MaxValue Step m_TickSmallFrequency
                    If m_TickLargeFrequency > 0 AndAlso _
                       (i - m_MinValue) Mod m_TickLargeFrequency = 0 Then
                        Continue For
                    End If
                    DrawLine(i, g, Scaled_TickSmallSize, p)
                Next
            End If
            ' ----------------------------------------------------------------- tiny ticks
            If m_TickTinyFrequency > 0 Then
                p = New Pen(ForeColor, Scaled_TickTinyWidth)
                For i = m_MinValue To m_MaxValue Step m_TickTinyFrequency
                    If (m_TickSmallFrequency > 0 AndAlso _
                       (i - m_MinValue) Mod m_TickSmallFrequency = 0) OrElse _
                       (m_TickLargeFrequency > 0 AndAlso _
                       (i - m_MinValue) Mod m_TickLargeFrequency = 0) Then
                        Continue For
                    End If
                    DrawLine(i, g, Scaled_TickTinySize, p)
                Next
            End If

        Else

            Dim j As Single = m_MinValue
            Do
                For i = 1 To 9
                    Dim v As Single = i * j
                    If v >= 1 Then
                        v = CSng(Math.Round(v))
                    End If
                    If i = 1 Then
                        p = New Pen(m_ScaleTextColor, Scaled_TickLargeWidth)
                        DrawLine(v, g, Scaled_TickLargeSize, p, True)
                        If j > m_MaxValue * 0.5 Then Exit Do
                    ElseIf i = 5 Then
                        p = New Pen(ForeColor, Scaled_TickSmallWidth)
                        DrawLine(v, g, Scaled_TickSmallSize, p)
                    Else
                        p = New Pen(ForeColor, Scaled_TickTinyWidth)
                        DrawLine(v, g, Scaled_TickTinySize, p)
                    End If
                Next
                j = j * 10
            Loop

        End If

        ' ----------------------------------------------------------------- finally, the title
        If ScaleText <> "" Then
            Dim sz As SizeF = g.MeasureString(ScaleText, Scaled_Font)
            g.DrawString(ScaleText, _
                         Scaled_Font, _
                         New SolidBrush(m_ScaleTextColor), _
                         m_framePadding.Left + (meterLocation.Width \ 2) - sz.Width / 2, _
                         m_framePadding.Top + CInt(meterLocation.Height * 0.8) - sz.Height / 2)
        End If

        g.Dispose()

        ' ----------------------------------------------------------------- set the background image
        If bgImage IsNot Nothing Then
            bgImage.Dispose()
        End If
        bgImage = bmp

        ' -----------------------------------------------------------------
        m_BackgroundChanged = False
        Me.Invalidate()
    End Sub



    ' ==========================================================================================================
    '  DRAW SCALE LINES
    ' ==========================================================================================================

    Private Const Krad As Single = CSng(Math.PI / 180)
    Private Shared angle1 As Single = 0
    Private Shared angle2 As Single = 0

    Private Sub DrawLine(ByVal value As Single, _
                         ByVal g As Graphics, _
                         ByVal TickSize As Single, _
                         ByVal _pen As Pen, _
                         Optional ByVal DrawText As Boolean = False, _
                         Optional ByVal Smoothing As Single = 0)

        'If value > m_MaxValue Then value = m_MaxValue
        'If value < m_MinValue Then value = m_MinValue

        Dim angle As Single = ValueToAngle(value)

        If Smoothing > 0 Then
            SmoothValue(angle1, angle, Smoothing)
            SmoothValue(angle2, angle1, Smoothing)
            angle = angle2
        End If

        If angle < m_TickStartAngle - 5 Then angle = m_TickStartAngle - 5
        If angle > m_TickEndAngle + 5 Then angle = m_TickEndAngle + 5


        Dim cosa As Double = Math.Cos(angle * Krad)
        Dim sina As Double = Math.Sin(angle * Krad)


        ' ---------------------------------------------------- p1 is outer point, p2 is inner point
        Dim p2x As Single
        Dim p2y As Single

        If TickSize = 0 Then
            p2x = 0
            p2y = 0
        Else
            p2x = p1x - TickSize
            p2y = p1y - TickSize
        End If

        Dim p(1) As PointF

        p(0) = New PointF(CSng((m_framePadding.Left + m_internalPadding.Left + (p1x - p1x * cosa))), _
                          CSng((m_framePadding.Top + m_internalPadding.Top + (p1y - p1y * sina))))

        p(1) = New PointF(CSng((m_framePadding.Left + m_internalPadding.Left + (p1x - p2x * cosa))), _
                          CSng((m_framePadding.Top + m_internalPadding.Top + (p1y - p2y * sina))))

        g.DrawLine(_pen, p(0), p(1))

        If DrawText Then

            Dim ci As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture
            Dim s As String
            If value >= 1000 Then
                s = (value / 1000.0F).ToString("0") & "K"
            ElseIf value > 0.9999 Or value < 0.0000001 Then
                s = value.ToString("0")
            ElseIf value > 0.0999 Then
                s = value.ToString("0.0", ci)
            ElseIf value > 0.00999 Then
                s = value.ToString("0.00", ci)
            Else
                s = value.ToString("0.000", ci)
            End If

            Dim sz As SizeF = g.MeasureString(s, Scaled_Font)

            'Dim p3x As Single = p2x - m_ScaleFont.Height
            Dim p3x As Single = p2x - sz.Width / 2
            Dim p3y As Single = p2y - Scaled_Font.Height

            p(1) = New PointF(CSng((m_framePadding.Left + m_internalPadding.Left + (p1x - p3x * cosa))), _
                              CSng((m_framePadding.Top + m_internalPadding.Top + (p1y - p3y * sina))))


            g.DrawString(s, _
                         Scaled_Font, _
                         _pen.Brush, _
                         p(1).X - sz.Width / 2, _
                         p(1).Y - sz.Height / 2)
        End If
    End Sub



    Private Function ValueToAngle(ByVal value As Single) As Single
        If mScaleLinear Then
            Return m_TickStartAngle + _
                   (180 - m_TickStartAngle * 2) * _
                   (value - m_MinValue) / _
                   (m_MaxValue - m_MinValue)
        Else
            If value > 0 Then
                Return m_TickStartAngle + _
                       (180 - m_TickStartAngle * 2) * _
                       CSng((Math.Log(value) - Math.Log(m_MinValue)) / _
                       (Math.Log(m_MaxValue) - Math.Log(m_MinValue)))
            End If
        End If
        Return m_TickStartAngle
    End Function




    ' ================================================================================================
    '   PROPS
    ' ================================================================================================

    Private m_Value As Single = 0.0F
    <DefaultValue(0.0F), Category("Meter"), Description("Value of meter")> _
    Public Property Value() As Single
        Get
            Return Me.m_Value
        End Get
        Set(ByVal value As Single)
            If value <> Me.m_Value Then
                Me.m_Value = value
                Me.Invalidate()
            End If
        End Set
    End Property

    Private m_ScaleText As String = ""
    <DefaultValue("Vac"), Category("Meter"), Description("Title to display on the meter")> _
    Public Property ScaleText() As String
        Get
            Return m_ScaleText
        End Get
        Set(ByVal value As String)
            m_ScaleText = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_MaxValue As Single = 500.0F
    <DefaultValue(500.0F), Category("Meter"), Description("Maximum value of the meter")> _
    Public Property MaxValue() As Single
        Get
            Return m_MaxValue
        End Get
        Set(ByVal value As Single)
            m_MaxValue = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_MinValue As Single = 0.0F
    <DefaultValue(0.0F), Category("Meter"), Description("Minimum value of the meter")> _
    Public Property MinValue() As Single
        Get
            Return m_MinValue
        End Get
        Set(ByVal value As Single)
            m_MinValue = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickTinyFrequency As Single = 10.0F
    <DefaultValue(10.0F), Category("Meter"), Description("Frequency of tiny ticks (0 to disable)")> _
    Public Property TickTinyFrequency() As Single
        Get
            Return m_TickTinyFrequency
        End Get
        Set(ByVal value As Single)
            m_TickTinyFrequency = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickSmallFrequency As Single = 50.0F
    <DefaultValue(50.0F), Category("Meter"), Description("Frequency of small ticks (0 to disable)")> _
    Public Property TickSmallFrequency() As Single
        Get
            Return m_TickSmallFrequency
        End Get
        Set(ByVal value As Single)
            m_TickSmallFrequency = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickLargeFrequency As Single = 100.0F
    <DefaultValue(100.0F), Category("Meter"), Description("Frequency of large ticks (0 to disable)")> _
    Public Property TickLargeFrequency() As Single
        Get
            Return m_TickLargeFrequency
        End Get
        Set(ByVal value As Single)
            m_TickLargeFrequency = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickTinyWidth As Single = 1.0F
    <DefaultValue(1.0F), Category("Meter"), Description("Stroke width of tiny ticks")> _
    Public Property TickTinyWidth() As Single
        Get
            Return m_TickTinyWidth
        End Get
        Set(ByVal value As Single)
            m_TickTinyWidth = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickSmallWidth As Single = 1.0F
    <DefaultValue(1.0F), Category("Meter"), Description("Stroke width of small ticks")> _
    Public Property TickSmallWidth() As Single
        Get
            Return m_TickSmallWidth
        End Get
        Set(ByVal value As Single)
            m_TickSmallWidth = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickLargeWidth As Single = 2.0F
    <DefaultValue(2.0F), Category("Meter"), Description("Stroke width of large ticks")> _
    Public Property TickLargeWidth() As Single
        Get
            Return m_TickLargeWidth
        End Get
        Set(ByVal value As Single)
            m_TickLargeWidth = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickStartAngle As Single = 20.0F
    <DefaultValue(20.0F), Category("Meter"), Description("Angle the meter starts display at in degrees")> _
    Public Property TickStartAngle() As Single
        Get
            Return m_TickStartAngle
        End Get
        Set(ByVal value As Single)
            m_TickStartAngle = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickEndAngle As Single = 160.0F
    <DefaultValue(160.0F), Category("Meter"), Description("Angle the meter end display at in degrees")> _
    Public Property TickEndAngle() As Single
        Get
            Return m_TickEndAngle
        End Get
        Set(ByVal value As Single)
            m_TickEndAngle = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickTinySize As Single = 5.0F
    <DefaultValue(5.0F), Category("Meter"), Description("Size of tiny ticks")> _
    Public Property TickTinySize() As Single
        Get
            Return m_TickTinySize
        End Get
        Set(ByVal value As Single)
            m_TickTinySize = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickSmallSize As Single = 15.0F
    <DefaultValue(15.0F), Category("Meter"), Description("Size of small ticks")> _
    Public Property TickSmallSize() As Single
        Get
            Return m_TickSmallSize
        End Get
        Set(ByVal value As Single)
            m_TickSmallSize = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_TickLargeSize As Single = 20.0F
    <DefaultValue(20.0F), Category("Meter"), Description("Size of large ticks")> _
    Public Property TickLargeSize() As Single
        Get
            Return m_TickLargeSize
        End Get
        Set(ByVal value As Single)
            m_TickLargeSize = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_ScaleFont As Font = New Font("Arial", 12)
    <Category("Meter"), Description("Scale text font")> _
    Public Property ScaleFont() As Font
        Get
            Return m_ScaleFont
        End Get
        Set(ByVal value As Font)
            m_ScaleFont = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_ScaleTextColor As Color = Color.Black
    <DefaultValue(GetType(Color), "Black"), Category("Meter"), Description("Scale text color")> _
    Public Property ScaleTextColor() As Color
        Get
            Return m_ScaleTextColor
        End Get
        Set(ByVal value As Color)
            m_ScaleTextColor = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private mScaleLinear As Boolean = True
    <DefaultValue(True), Category("Meter"), Description("Meter scale type")> _
    Public Property LinearScale() As Boolean
        Get
            Return mScaleLinear
        End Get
        Set(ByVal value As Boolean)
            mScaleLinear = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_PointerWidth As Single = 3.0F
    <DefaultValue(3.0F), Category("Meter"), Description("Stroke width of pointer")> _
    Public Property PointerWidth() As Single
        Get
            Return m_PointerWidth
        End Get
        Set(ByVal value As Single)
            m_PointerWidth = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_PointerColor As Color = Color.Black
    <DefaultValue(GetType(Color), "Black"), Category("Meter"), Description("Color of the primary pointer")> _
    Public Property PointerColor() As Color
        Get
            Return m_PointerColor
        End Get
        Set(ByVal value As Color)
            m_PointerColor = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_framePadding As New Padding(5)
    <Category("Meter"), Description("Padding ...")> _
    Public Property FramePadding() As Padding
        Get
            Return m_framePadding
        End Get
        Set(ByVal value As Padding)
            m_framePadding = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property

    Private m_internalPadding As New Padding(5)
    <Category("Meter"), Description("Padding ...")> _
    Public Property InternalPadding() As Padding
        Get
            Return m_internalPadding
        End Get
        Set(ByVal value As Padding)
            m_internalPadding = value
            m_BackgroundChanged = True
            Me.Invalidate()
        End Set
    End Property


End Class
